@@ -18,14 +18,13 @@ module Annex.Branch (
 	get,
 	change,
 	commit,
-	stage,
 	files,
 ) where
 
 import qualified Data.ByteString.Lazy.Char8 as L
+import System.Environment
 
 import Common.Annex
-import Annex.Exception
 import Annex.BranchState
 import Annex.Journal
 import qualified Git
@@ -37,9 +36,9 @@ import qualified Git.UpdateIndex
 import Git.HashObject
 import Git.Types
 import Git.FilePath
-import qualified Git.Index
 import Annex.CatFile
 import Annex.Perms
+import qualified Annex
 
 {- Name of the branch that is used to store git-annex's information. -}
 name :: Git.Ref
@@ -86,43 +85,53 @@ getBranch = maybe (hasOrigin >>= go >>= use) return =<< branchsha
 			return sha
 		branchsha = inRepo $ Git.Ref.sha fullname
 
-{- Ensures that the branch and index are is up-to-date; should be
- - called before data is read from it. Runs only once per git-annex run.
- -}
+{- Ensures that the branch and index are up-to-date; should be
+ - called before data is read from it. Runs only once per git-annex run. -}
 update :: Annex ()
-update = runUpdateOnce $ updateTo =<< siblingBranches
+update = runUpdateOnce $ void $ updateTo =<< siblingBranches
 
 {- Forces an update even if one has already been run. -}
-forceUpdate :: Annex ()
+forceUpdate :: Annex Bool
 forceUpdate = updateTo =<< siblingBranches
 
 {- Merges the specified Refs into the index, if they have any changes not
  - already in it. The Branch names are only used in the commit message;
  - it's even possible that the provided Branches have not been updated to
  - point to the Refs yet.
- -
- - Before refs are merged into the index, it's important to first stage the
- - journal into the index. Otherwise, any changes in the journal would
- - later get staged, and might overwrite changes made during the merge.
- - If no Refs are provided, the journal is still staged and committed.
- -
- - (It would be cleaner to handle the merge by updating the journal, not the
- - index, with changes from the branches.)
- -
+ - 
  - The branch is fast-forwarded if possible, otherwise a merge commit is
  - made.
+ -
+ - Before Refs are merged into the index, it's important to first stage the
+ - journal into the index. Otherwise, any changes in the journal would
+ - later get staged, and might overwrite changes made during the merge.
+ - This is only done if some of the Refs do need to be merged.
+ -
+ - Returns True if any refs were merged in, False otherwise.
  -}
-updateTo :: [(Git.Ref, Git.Branch)] -> Annex ()
+updateTo :: [(Git.Ref, Git.Branch)] -> Annex Bool
 updateTo pairs = do
 	-- ensure branch exists, and get its current ref
 	branchref <- getBranch
-	-- check what needs updating before taking the lock
-	dirty <- unCommitted
+	dirty <- journalDirty
 	(refs, branches) <- unzip <$> filterM isnewer pairs
-	if not dirty && null refs
-		then updateIndex branchref
-		else withIndex $ lockJournal $ do
-			when dirty stageJournal
+	if null refs
+ 		{- Even when no refs need to be merged, the index
+		 - may still be updated if the branch has gotten ahead 
+		 - of the index. -}
+		then whenM (needUpdateIndex branchref) $ lockJournal $ do
+			forceUpdateIndex branchref
+			{- When there are journalled changes
+			 - as well as the branch being updated,
+			 - a commit needs to be done. -}
+			when dirty $
+				go branchref True [] []
+		else lockJournal $ go branchref dirty refs branches
+	return $ not $ null refs
+	where
+		isnewer (r, _) = inRepo $ Git.Branch.changed fullname r
+		go branchref dirty refs branches = withIndex $ do
+			cleanjournal <- if dirty then stageJournal else return noop
 			let merge_desc = if null branches
 				then "update"
 				else "merging " ++
@@ -139,11 +148,14 @@ updateTo pairs = do
 				else commitBranch branchref merge_desc
 					(nub $ fullname:refs)
 			invalidateCache
-	where
-		isnewer (r, _) = inRepo $ Git.Branch.changed fullname r
+			liftIO cleanjournal
 
-{- Gets the content of a file on the branch, or content from the journal, or
- - staged in the index.
+{- Gets the content of a file, which may be in the journal, or committed
+ - to the branch. Due to limitatons of git cat-file, does *not* get content
+ - that has only been staged to the index.
+ - 
+ - Updates the branch if necessary, to ensure the most up-to-date available
+ - content is available.
  -
  - Returns an empty string if the file doesn't exist yet. -}
 get :: FilePath -> Annex String
@@ -163,7 +175,9 @@ get' staleok file = fromcache =<< getCache file
 		fromjournal (Just content) = cache content
 		fromjournal Nothing
 			| staleok = withIndex frombranch
-			| otherwise = withIndexUpdate $ frombranch >>= cache
+			| otherwise = do
+				update
+				withIndex $ frombranch >>= cache
 		frombranch = L.unpack <$> catFile fullname file
 		cache content = do
 			setCache file content
@@ -185,16 +199,11 @@ set file content = do
 
 {- Stages the journal, and commits staged changes to the branch. -}
 commit :: String -> Annex ()
-commit message = whenM unCommitted $ lockJournal $ do
-	stageJournal
+commit message = whenM journalDirty $ lockJournal $ do
+	cleanjournal <- stageJournal
 	ref <- getBranch
 	withIndex $ commitBranch ref message [fullname]
-
-{- Stages the journal, not making a commit to the branch. -}
-stage :: Annex ()
-stage = whenM journalDirty $ lockJournal $ do
-	stageJournal
-	setUnCommitted
+	liftIO $ cleanjournal
 
 {- Commits the staged changes in the index to the branch.
  - 
@@ -216,13 +225,16 @@ stage = whenM journalDirty $ lockJournal $ do
  -}
 commitBranch :: Git.Ref -> String -> [Git.Ref] -> Annex ()
 commitBranch branchref message parents = do
+	showStoringStateAction
+	commitBranch' branchref message parents
+commitBranch' :: Git.Ref -> String -> [Git.Ref] -> Annex ()
+commitBranch' branchref message parents = do
 	updateIndex branchref
 	committedref <- inRepo $ Git.Branch.commit message fullname parents
 	setIndexSha committedref
 	parentrefs <- commitparents <$> catObject committedref
 	when (racedetected branchref parentrefs) $
 		fixrace committedref parentrefs
-	setCommitted
 	where
 		-- look for "parent ref" lines and return the refs
 		commitparents = map (Git.Ref . snd) . filter isparent .
@@ -247,12 +259,15 @@ commitBranch branchref message parents = do
 
 {- Lists all files on the branch. There may be duplicates in the list. -}
 files :: Annex [FilePath]
-files = withIndexUpdate $ do
-	bfiles <- inRepo $ Git.Command.pipeNullSplit
-		[Params "ls-tree --name-only -r -z", Param $ show fullname]
-	jfiles <- getJournalledFiles
-	return $ jfiles ++ bfiles
-
+files = do
+	update
+	withIndex $ do
+		bfiles <- inRepo $ Git.Command.pipeNullSplitZombie
+			[ Params "ls-tree --name-only -r -z"
+			, Param $ show fullname
+			]
+		jfiles <- getJournalledFiles
+		return $ jfiles ++ bfiles
 
 {- Populates the branch's index file with the current branch contents.
  - 
@@ -277,17 +292,19 @@ withIndex = withIndex' False
 withIndex' :: Bool -> Annex a -> Annex a
 withIndex' bootstrapping a = do
 	f <- fromRepo gitAnnexIndex
-	bracketIO (Git.Index.override f) id $ do
-		checkIndexOnce $ unlessM (liftIO $ doesFileExist f) $ do
-			unless bootstrapping create
-			liftIO $ createDirectoryIfMissing True $ takeDirectory f
-			unless bootstrapping $ inRepo genIndex
-		a
+	g <- gitRepo
+	e <- liftIO getEnvironment
+	let g' = g { gitEnv = Just $ ("GIT_INDEX_FILE", f):e }
 
-{- Runs an action using the branch's index file, first making sure that
- - the branch and index are up-to-date. -}
-withIndexUpdate :: Annex a -> Annex a
-withIndexUpdate a = update >> withIndex a
+	Annex.changeState $ \s -> s { Annex.repo = g' }
+	checkIndexOnce $ unlessM (liftIO $ doesFileExist f) $ do
+		unless bootstrapping create
+		liftIO $ createDirectoryIfMissing True $ takeDirectory f
+		unless bootstrapping $ inRepo genIndex
+	r <- a
+	Annex.changeState $ \s -> s { Annex.repo = (Annex.repo s) { gitEnv = gitEnv g} }
+
+	return r
 
 {- Updates the branch's index to reflect the current contents of the branch.
  - Any changes staged in the index will be preserved.
@@ -296,13 +313,21 @@ withIndexUpdate a = update >> withIndex a
  - ref of the branch to see if an update is needed.
  -}
 updateIndex :: Git.Ref -> Annex ()
-updateIndex branchref = do
+updateIndex branchref = whenM (needUpdateIndex branchref) $
+	forceUpdateIndex branchref
+
+forceUpdateIndex :: Git.Ref -> Annex ()
+forceUpdateIndex branchref = do
+	withIndex $ mergeIndex [fullname]
+	setIndexSha branchref
+
+{- Checks if the index needs to be updated. -}
+needUpdateIndex :: Git.Ref -> Annex Bool
+needUpdateIndex branchref = do
 	lock <- fromRepo gitAnnexIndexLock
 	lockref <- Git.Ref . firstLine <$>
-		liftIO (catchDefaultIO (readFileStrict lock) "")
-	when (lockref /= branchref) $ do
-		withIndex $ mergeIndex [fullname]
-		setIndexSha branchref
+		liftIO (catchDefaultIO "" $ readFileStrict lock)
+	return (lockref /= branchref)
 
 {- Record that the branch's index has been updated to correspond to a
  - given ref of the branch. -}
@@ -312,39 +337,24 @@ setIndexSha ref = do
 	liftIO $ writeFile lock $ show ref ++ "\n"
 	setAnnexPerm lock
 
-{- Checks if there are uncommitted changes in the branch's index or journal. -}
-unCommitted :: Annex Bool
-unCommitted = do
-	d <- liftIO . doesFileExist =<< fromRepo gitAnnexIndexDirty
-	if d
-		then return d
-		else journalDirty
-
-setUnCommitted :: Annex ()
-setUnCommitted = do
-	file <- fromRepo gitAnnexIndexDirty
-	liftIO $ writeFile file "1"
-
-setCommitted :: Annex ()
-setCommitted = void $ do
-	file <- fromRepo gitAnnexIndexDirty
-	liftIO $ tryIO $ removeFile file
-
-{- Stages the journal into the index. -}
-stageJournal :: Annex ()
-stageJournal = do
-	showStoringStateAction
-	fs <- getJournalFiles
+{- Stages the journal into the index and returns an action that will
+ - clean up the staged journal files, which should only be run once
+ - the index has been committed to the branch. Should be run within
+ - lockJournal, to prevent others from modifying the journal. -}
+stageJournal :: Annex (IO ())
+stageJournal = withIndex $ do
 	g <- gitRepo
-	withIndex $ liftIO $ do
+	let dir = gitAnnexJournalDir g
+	fs <- getJournalFiles
+	liftIO $ do
 		h <- hashObjectStart g
 		Git.UpdateIndex.streamUpdateIndex g
-			[genstream (gitAnnexJournalDir g) h fs]
+			[genstream dir h fs]
 		hashObjectStop h
+	return $ liftIO $ mapM_ removeFile $ map (dir </>) fs
 	where
 		genstream dir h fs streamer = forM_ fs $ \file -> do
 			let path = dir </> file
 			sha <- hashFile h path
-			_ <- streamer $ Git.UpdateIndex.updateIndexLine
+			streamer $ Git.UpdateIndex.updateIndexLine
 				sha FileBlob (asTopFilePath $ fileJournal file)
-			removeFile path
