@@ -5,6 +5,8 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Annex.Ssh (
 	sshParams,
 	sshCleanup,
@@ -14,7 +16,7 @@ import qualified Data.Map as M
 
 import Common.Annex
 import Annex.LockPool
-import qualified Git
+import qualified Git.Config
 import Config
 import qualified Build.SysConfig as SysConfig
 import Annex.Perms
@@ -42,13 +44,23 @@ sshInfo (host, port) = ifM caching
 	( do
 		dir <- fromRepo gitAnnexSshDir
 		let socketfile = dir </> hostport2socket host port
-	 	return (Just socketfile, cacheParams socketfile)
+		if valid_unix_socket_path socketfile
+			then return (Just socketfile, cacheParams socketfile)
+			else do
+				socketfile' <- liftIO $ relPathCwdToFile socketfile
+				if valid_unix_socket_path socketfile'
+					then return (Just socketfile', cacheParams socketfile')
+					else return (Nothing, [])
 	, return (Nothing, [])
 	)
 	where
+#ifdef WITH_OLD_SSH
+		caching = return False
+#else
 		caching = fromMaybe SysConfig.sshconnectioncaching 
-			. Git.configTrue
+			. Git.Config.isTrue
 			<$> getConfig (annexConfig "sshcaching") ""
+#endif
 
 cacheParams :: FilePath -> [CommandParam]
 cacheParams socketfile =
@@ -65,7 +77,7 @@ sshCleanup :: Annex ()
 sshCleanup = do
 	dir <- fromRepo gitAnnexSshDir
 	sockets <- filter (not . isLock) <$>
-		liftIO (catchDefaultIO (dirContents dir) [])
+		liftIO (catchDefaultIO [] $ dirContents dir)
 	forM_ sockets cleanup
 	where
 		cleanup socketfile = do
@@ -118,3 +130,16 @@ isLock f = lockExt `isSuffixOf` f
 
 lockExt :: String
 lockExt = ".lock"
+
+{- This is the size of the sun_path component of sockaddr_un, which
+ - is the limit to the total length of the filename of a unix socket.
+ -
+ - On Linux, this is 108. On OSX, 104. TODO: Probe
+ -}
+sizeof_sockaddr_un_sun_path :: Int
+sizeof_sockaddr_un_sun_path = 100
+
+{- Note that this looks at the true length of the path in bytes, as it will
+ - appear on disk. -}
+valid_unix_socket_path :: FilePath -> Bool
+valid_unix_socket_path f = length (decodeW8 f) < sizeof_sockaddr_un_sun_path
