@@ -32,14 +32,11 @@ configkey = annexConfig "uuid"
 {- Generates a UUID. There is a library for this, but it's not packaged,
  - so use the command line tool. -}
 genUUID :: IO UUID
-genUUID = pOpen ReadFromPipe command params $ liftM toUUID . hGetLine
+genUUID = gen . lines <$> readProcess command params
 	where
-		command = SysConfig.uuid
-		params
-			-- request a random uuid be generated
-			| command == "uuid" = ["-m"]
-			-- uuidgen generates random uuid by default
-			| otherwise = []
+		gen [] = error $ "no output from " ++ command
+		gen (l:_) = toUUID l
+		(command:params) = words SysConfig.uuid
 
 {- Get current repository's UUID. -}
 getUUID :: Annex UUID
