@@ -4,7 +4,7 @@ module Build.Configure where
 
 import System.Directory
 import Data.List
-import System.Cmd.Utils
+import System.Process
 import Control.Applicative
 import System.FilePath
 
@@ -19,23 +19,23 @@ tests =
 	, testCp "cp_a" "-a"
 	, testCp "cp_p" "-p"
 	, testCp "cp_reflink_auto" "--reflink=auto"
-	, TestCase "uuid generator" $ selectCmd "uuid" ["uuid", "uuidgen"] ""
+	, TestCase "uuid generator" $ selectCmd "uuid" ["uuid -m", "uuid", "uuidgen"] ""
 	, TestCase "xargs -0" $ requireCmd "xargs_0" "xargs -0 </dev/null"
 	, TestCase "rsync" $ requireCmd "rsync" "rsync --version >/dev/null"
 	, TestCase "curl" $ testCmd "curl" "curl --version >/dev/null"
 	, TestCase "wget" $ testCmd "wget" "wget --version >/dev/null"
 	, TestCase "bup" $ testCmd "bup" "bup --version >/dev/null"
 	, TestCase "gpg" $ testCmd "gpg" "gpg --version >/dev/null"
+	, TestCase "lsof" $ testCmd "lsof" "lsof -v >/dev/null 2>&1"
 	, TestCase "ssh connection caching" getSshConnectionCaching
-	] ++ shaTestCases False [1, 512, 224, 384] ++ shaTestCases True [256]
+	] ++ shaTestCases [1, 256, 512, 224, 384]
 
-shaTestCases :: Bool -> [Int] -> [TestCase]
-shaTestCases required l = map make l
+shaTestCases :: [Int] -> [TestCase]
+shaTestCases l = map make l
 	where
-		make n = TestCase key $ selector key (shacmds n) "</dev/null"
+		make n = TestCase key $ maybeSelectCmd key (shacmds n) "</dev/null"
 			where
 				key = "sha" ++ show n
-		selector = if required then selectCmd else maybeSelectCmd
 		shacmds n = concatMap (\x -> [x, osxpath </> x]) $
 			map (\x -> "sha" ++ show n ++ x) ["", "sum"]
 		-- Max OSX puts GNU tools outside PATH, so look in
@@ -71,8 +71,8 @@ getVersionString = do
 
 getGitVersion :: Test
 getGitVersion = do
-	(_, s) <- pipeFrom "git" ["--version"]
-	let version = last $ words $ head $ lines s
+	s <- readProcess "git" ["--version"] ""
+	let version = unwords $ drop 2 $ words $ head $ lines s
 	return $ Config "gitversion" (StringConfig version)
 
 getSshConnectionCaching :: Test
