@@ -15,12 +15,12 @@ import qualified Control.Exception as E
 import qualified Data.Map as M
 import Control.Exception (throw)
 import System.Console.GetOpt
+import System.Posix.Signals
 
 import Common.Annex
 import qualified Annex
 import qualified Annex.Queue
 import qualified Git
-import qualified Git.Command
 import qualified Git.AutoCorrect
 import Annex.Content
 import Annex.Ssh
@@ -30,8 +30,8 @@ type Params = [String]
 type Flags = [Annex ()]
 
 {- Runs the passed command line. -}
-dispatch :: Bool -> Params -> [Command] -> [Option] -> String -> IO Git.Repo -> IO ()
-dispatch fuzzyok allargs allcmds commonoptions header getgitrepo = do
+dispatch :: Bool -> Params -> [Command] -> [Option] -> [(String, String)] -> String -> IO Git.Repo -> IO ()
+dispatch fuzzyok allargs allcmds commonoptions fields header getgitrepo = do
 	setupConsole
 	r <- E.try getgitrepo :: IO (Either E.SomeException Git.Repo)
 	case r of
@@ -40,9 +40,10 @@ dispatch fuzzyok allargs allcmds commonoptions header getgitrepo = do
 			state <- Annex.new g
 			(actions, state') <- Annex.run state $ do
 				checkfuzzy
+				forM_ fields $ \(f, v) -> Annex.setField f v
 				sequence_ flags
 				prepCommand cmd params
-		 	tryRun state' cmd $ [startup] ++ actions ++ [shutdown $ cmdoneshot cmd]
+		 	tryRun state' cmd $ [startup] ++ actions ++ [shutdown $ cmdnocommit cmd]
 	where
 		err msg = msg ++ "\n\n" ++ usage header allcmds commonoptions
 		cmd = Prelude.head cmds
@@ -88,7 +89,7 @@ tryRun = tryRun' 0
 tryRun' :: Integer -> Annex.AnnexState -> Command -> [CommandCleanup] -> IO ()
 tryRun' errnum _ cmd []
 	| errnum > 0 = error $ cmdname cmd ++ ": " ++ show errnum ++ " failed"
-	| otherwise = return ()
+	| otherwise = noop
 tryRun' errnum state cmd (a:as) = do
 	r <- run
 	handle $! r
@@ -107,13 +108,14 @@ tryRun' errnum state cmd (a:as) = do
 
 {- Actions to perform each time ran. -}
 startup :: Annex Bool
-startup = return True
+startup = liftIO $ do
+	void $ installHandler sigINT Default Nothing
+	return True
 
 {- Cleanup actions. -}
 shutdown :: Bool -> Annex Bool
-shutdown oneshot = do
-	saveState oneshot
+shutdown nocommit = do
+	saveState nocommit
 	sequence_ =<< M.elems <$> Annex.getState Annex.cleanup
-	liftIO Git.Command.reap -- zombies from long-running git processes
 	sshCleanup -- ssh connection caching
 	return True
