@@ -8,7 +8,7 @@
 module Command (
 	command,
 	noRepo,
-	oneShot,
+	noCommit,
 	withOptions,
 	next,
 	stop,
@@ -21,6 +21,8 @@ module Command (
 	isBareRepo,
 	numCopies,
 	autoCopies,
+	autoCopiesWith,
+	checkAuto,
 	module ReExported
 ) where
 
@@ -37,14 +39,16 @@ import Usage as ReExported
 import Logs.Trust
 import Config
 import Annex.CheckAttr
+import qualified Git.Command
 
 {- Generates a normal command -}
 command :: String -> String -> [CommandSeek] -> String -> Command
 command = Command [] Nothing commonChecks False
 
-{- Makes a command run in oneshot mode. -}
-oneShot :: Command -> Command
-oneShot c = c { cmdoneshot = True }
+{- Indicates that a command doesn't need to commit any changes to
+ - the git-annex branch. -}
+noCommit :: Command -> Command
+noCommit c = c { cmdnocommit = True }
 
 {- Adds a fallback action to a command, that will be run if it's used
  - outside a git repository. -}
@@ -80,11 +84,14 @@ doCommand = start
 	where
 		start   = stage $ maybe skip perform
 		perform = stage $ maybe failure cleanup
-		cleanup = stage $ status
+		cleanup = stage $ end
 		stage = (=<<)
 		skip = return True
 		failure = showEndFail >> return False
-		status r = showEndResult r >> return r
+		end r = do
+			-- zombies from long-running git processes
+			liftIO Git.Command.reap
+			showEndResult r >> return r
 
 {- Modifies an action to only act on files that are already annexed,
  - and passes the key and backend on to it. -}
@@ -111,9 +118,20 @@ numCopies file = readish <$> checkAttr "annex.numcopies" file
  -
  - In auto mode, first checks that the number of known
  - copies of the key is > or < than the numcopies setting, before running
- - the action. -}
-autoCopies :: FilePath -> Key -> (Int -> Int -> Bool) -> (Maybe Int -> CommandStart) -> CommandStart
-autoCopies file key vs a = do
+ - the action. Also checks any preferred content settings.
+ -}
+autoCopies :: FilePath -> Key -> (Int -> Int -> Bool) -> CommandStart -> CommandStart
+autoCopies file key vs a = Annex.getState Annex.auto >>= go
+	where
+		go False = a
+		go True = do
+			numcopiesattr <- numCopies file
+			needed <- getNumCopies numcopiesattr
+			(_, have) <- trustPartition UnTrusted =<< Remote.keyLocations key
+			if length have `vs` needed then a else stop
+
+autoCopiesWith :: FilePath -> Key -> (Int -> Int -> Bool) -> (Maybe Int -> CommandStart) -> CommandStart
+autoCopiesWith file key vs a = do
 	numcopiesattr <- numCopies file
 	Annex.getState Annex.auto >>= auto numcopiesattr
 	where
@@ -121,4 +139,10 @@ autoCopies file key vs a = do
 		auto numcopiesattr True = do
 			needed <- getNumCopies numcopiesattr
 			(_, have) <- trustPartition UnTrusted =<< Remote.keyLocations key
-			if length have `vs` needed then a numcopiesattr else stop
+			if length have `vs` needed
+				then a numcopiesattr
+				else stop
+
+checkAuto :: Annex Bool -> Annex Bool
+checkAuto checker = ifM (Annex.getState Annex.auto)
+	( checker , return True )
