@@ -17,6 +17,7 @@ import Logs.Trust
 import Annex.Content
 import Config
 import qualified Option
+import Annex.Wanted
 
 def :: [Command]
 def = [withOptions [fromOption] $ command "drop" paramPaths seek
@@ -30,14 +31,15 @@ seek = [withField fromOption Remote.byName $ \from ->
 	withFilesInGit $ whenAnnexed $ start from]
 
 start :: Maybe Remote -> FilePath -> (Key, Backend) -> CommandStart
-start from file (key, _) = autoCopies file key (>) $ \numcopies ->
-	case from of
-		Nothing -> startLocal file numcopies key
-		Just remote -> do
-			u <- getUUID
-			if Remote.uuid remote == u
-				then startLocal file numcopies key
-				else startRemote file numcopies key remote
+start from file (key, _) = autoCopiesWith file key (>) $ \numcopies ->
+	stopUnless (checkAuto $ wantDrop (Remote.uuid <$> from) (Just file)) $
+		case from of
+			Nothing -> startLocal file numcopies key
+			Just remote -> do
+				u <- getUUID
+				if Remote.uuid remote == u
+					then startLocal file numcopies key
+					else startRemote file numcopies key remote
 
 startLocal :: FilePath -> Maybe Int -> Key -> CommandStart
 startLocal file numcopies key = stopUnless (inAnnex key) $ do
