@@ -12,9 +12,10 @@ import Command
 import qualified Annex
 import Logs.Location
 import Annex.Content
+import Types.Key
 
 def :: [Command]
-def = [command "dropkey" (paramRepeating paramKey) seek
+def = [noCommit $ command "dropkey" (paramRepeating paramKey) seek
 	"drops annexed content for specified keys"] 
 
 seek :: [CommandSeek]
@@ -24,7 +25,7 @@ start :: Key -> CommandStart
 start key = stopUnless (inAnnex key) $ do
 	unlessM (Annex.getState Annex.force) $
 		error "dropkey can cause data loss; use --force if you're sure you want to do this"
-	showStart "dropkey" (show key)
+	showStart "dropkey" (key2file key)
 	next $ perform key
 
 perform :: Key -> CommandPerform
