@@ -24,6 +24,14 @@ import Utility.DataUnits
 import Utility.FileMode
 import Config
 import qualified Option
+import Types.Key
+import Utility.HumanTime
+
+import System.Posix.Process (getProcessID)
+import Data.Time.Clock.POSIX
+import Data.Time
+import System.Posix.Types (EpochTime)
+import System.Locale
 
 def :: [Command]
 def = [withOptions options $ command "fsck" paramPaths seek
@@ -32,25 +40,71 @@ def = [withOptions options $ command "fsck" paramPaths seek
 fromOption :: Option
 fromOption = Option.field ['f'] "from" paramRemote "check remote"
 
+startIncrementalOption :: Option
+startIncrementalOption = Option.flag ['S'] "incremental" "start an incremental fsck"
+
+moreIncrementalOption :: Option
+moreIncrementalOption = Option.flag ['m'] "more" "continue an incremental fsck"
+
+incrementalScheduleOption :: Option
+incrementalScheduleOption = Option.field [] "incremental-schedule" paramTime
+	"schedule incremental fscking"
+
 options :: [Option]
-options = [fromOption]
+options = 
+	[ fromOption
+	, startIncrementalOption
+	, moreIncrementalOption
+	, incrementalScheduleOption
+	]
 
 seek :: [CommandSeek]
 seek =
 	[ withField fromOption Remote.byName $ \from ->
-		withFilesInGit $ whenAnnexed $ start from
-	, withBarePresentKeys startBare
+	  withIncremental $ \i -> withFilesInGit $ whenAnnexed $ start from i
+	, withIncremental $ \i -> withBarePresentKeys $ startBare i
 	]
 
-start :: Maybe Remote -> FilePath -> (Key, Backend) -> CommandStart
-start from file (key, backend) = do
-	numcopies <- numCopies file
-	showStart "fsck" file
-	case from of
-		Nothing -> next $ perform key file backend numcopies
-		Just r -> next $ performRemote key file backend numcopies r
+withIncremental :: (Incremental -> CommandSeek) -> CommandSeek
+withIncremental = withValue $ do
+	i <- maybe (return False) (checkschedule . parseDuration)
+		=<< Annex.getField (Option.name incrementalScheduleOption)
+	starti <- Annex.getFlag (Option.name startIncrementalOption)
+	morei <- Annex.getFlag (Option.name moreIncrementalOption)
+	case (i, starti, morei) of
+		(False, False, False) -> return NonIncremental
+		(False, True, _) -> startIncremental
+		(False ,False, True) -> ContIncremental <$> getStartTime
+		(True, _, _) ->
+			maybe startIncremental (return . ContIncremental . Just)
+				=<< getStartTime
+	where
+		startIncremental = do
+			recordStartTime
+			return StartIncremental
 
-perform :: Key -> FilePath -> Backend -> Maybe Int -> CommandPerform
+		checkschedule Nothing = error "bad --incremental-schedule value"
+		checkschedule (Just delta) = do
+			Annex.addCleanup "" $ do
+				v <- getStartTime
+				case v of
+					Nothing -> noop
+					Just started -> do
+						now <- liftIO getPOSIXTime
+						when (now - realToFrac started >= delta) $
+							resetStartTime
+			return True
+
+start :: Maybe Remote -> Incremental -> FilePath -> (Key, Backend) -> CommandStart
+start from inc file (key, backend) = do
+	numcopies <- numCopies file
+	case from of
+		Nothing -> go $ perform key file backend numcopies
+		Just r -> go $ performRemote key file backend numcopies r
+	where
+		go = runFsck inc file key
+
+perform :: Key -> FilePath -> Backend -> Maybe Int -> Annex Bool
 perform key file backend numcopies = check
 	-- order matters
 	[ fixLink key file
@@ -62,13 +116,13 @@ perform key file backend numcopies = check
 
 {- To fsck a remote, the content is retrieved to a tmp file,
  - and checked locally. -}
-performRemote :: Key -> FilePath -> Backend -> Maybe Int -> Remote -> CommandPerform
+performRemote :: Key -> FilePath -> Backend -> Maybe Int -> Remote -> Annex Bool
 performRemote key file backend numcopies remote =
 	dispatch =<< Remote.hasKey remote key
 	where
 		dispatch (Left err) = do
 			showNote err
-			stop
+			return False
 		dispatch (Right True) = withtmp $ \tmpfile ->
 			ifM (getfile tmpfile)
 				( go True (Just tmpfile)
@@ -94,7 +148,7 @@ performRemote key file backend numcopies remote =
 				( return True
 				, ifM (Annex.getState Annex.fast)
 					( return False
-					, Remote.retrieveKeyFile remote key tmp
+					, Remote.retrieveKeyFile remote key Nothing tmp
 					)
 				)
 
@@ -108,30 +162,23 @@ withBarePresentKeys a params = isBareRepo >>= go
 				error "fsck should be run without parameters in a bare repository"
 			map a <$> loggedKeys
 
-startBare :: Key -> CommandStart
-startBare key = case Backend.maybeLookupBackendName (Types.Key.keyBackendName key) of
+startBare :: Incremental -> Key -> CommandStart
+startBare inc key = case Backend.maybeLookupBackendName (Types.Key.keyBackendName key) of
 	Nothing -> stop
-	Just backend -> do
-		showStart "fsck" (show key)
-		next $ performBare key backend
+	Just backend -> runFsck inc (key2file key) key $ performBare key backend
 
 {- Note that numcopies cannot be checked in a bare repository, because
  - getting the numcopies value requires a working copy with .gitattributes
  - files. -}
-performBare :: Key -> Backend -> CommandPerform
+performBare :: Key -> Backend -> Annex Bool
 performBare key backend = check
-	[ verifyLocationLog key (show key)
+	[ verifyLocationLog key (key2file key)
 	, checkKeySize key
 	, checkBackend backend key
 	]
 
-check :: [Annex Bool] -> CommandPerform	
-check = sequence >=> dispatch
-	where
-		dispatch vs
-			| all (== True) vs = next $ return True
-			| otherwise = stop
-
+check :: [Annex Bool] -> Annex Bool
+check cs = all id <$> sequence cs
 
 {- Checks that the file's symlink points correctly to the content. -}
 fixLink :: Key -> FilePath -> Annex Bool
@@ -145,17 +192,17 @@ fixLink key file = do
 		 -}
 		whenM (liftIO $ doesFileExist file) $
 			unlessM (inAnnex key) $ do
-				showNote $ "fixing content location"
+				showNote "fixing content location"
 				dir <- liftIO $ parentDir <$> absPath file
 				let content = absPathFrom dir have
 				liftIO $ allowWrite (parentDir content)
 				moveAnnex key content
 
-		showNote $ "fixing link"
+		showNote "fixing link"
 		liftIO $ createDirectoryIfMissing True (parentDir file)
 		liftIO $ removeFile file
 		liftIO $ createSymbolicLink want file
-		Annex.Queue.add "add" [Param "--force", Param "--"] [file]
+		Annex.Queue.addCommand "add" [Param "--force", Param "--"] [file]
 	return True
 
 {- Checks that the location log reflects the current status of the key,
@@ -206,21 +253,21 @@ checkKeySize :: Key -> Annex Bool
 checkKeySize key = do
 	file <- inRepo $ gitAnnexLocation key
 	ifM (liftIO $ doesFileExist file)
-		( checkKeySize' key file badContent
+		( checkKeySizeOr badContent key file
 		, return True
 		)
 
 checkKeySizeRemote :: Key -> Remote -> Maybe FilePath -> Annex Bool
 checkKeySizeRemote _ _ Nothing = return True
-checkKeySizeRemote key remote (Just file) = checkKeySize' key file
-	(badContentRemote remote)
+checkKeySizeRemote key remote (Just file) =
+	checkKeySizeOr (badContentRemote remote) key file
 
-checkKeySize' :: Key -> FilePath -> (Key -> Annex String) -> Annex Bool
-checkKeySize' key file bad = case Types.Key.keySize key of
+checkKeySizeOr :: (Key -> Annex String) -> Key -> FilePath -> Annex Bool
+checkKeySizeOr bad key file = case Types.Key.keySize key of
 	Nothing -> return True
 	Just size -> do
 		size' <- fromIntegral . fileSize
-			<$> (liftIO $ getFileStatus file)
+			<$> liftIO (getFileStatus file)
 		comparesizes size size'
 	where
 		comparesizes a b = do
@@ -239,22 +286,23 @@ checkKeySize' key file bad = case Types.Key.keySize key of
 checkBackend :: Backend -> Key -> Annex Bool
 checkBackend backend key = do
 	file <- inRepo (gitAnnexLocation key)
-	checkBackend' backend key (Just file) badContent
+	checkBackendOr badContent backend key file
 
 checkBackendRemote :: Backend -> Key -> Remote -> Maybe FilePath -> Annex Bool
-checkBackendRemote backend key remote localcopy =
-	checkBackend' backend key localcopy (badContentRemote remote)
+checkBackendRemote backend key remote = maybe (return True) go
+	where
+		go = checkBackendOr (badContentRemote remote) backend key
 
-checkBackend' :: Backend -> Key -> Maybe FilePath -> (Key -> Annex String) -> Annex Bool
-checkBackend' _ _ Nothing _ = return True
-checkBackend' backend key (Just file) bad = case Types.Backend.fsckKey backend of
-	Nothing -> return True
-	Just a -> do
-		ok <- a key file
-		unless ok $ do
-			msg <- bad key
-			warning $ "Bad file content; " ++ msg
-		return ok
+checkBackendOr :: (Key -> Annex String) -> Backend -> Key -> FilePath -> Annex Bool
+checkBackendOr bad backend key file =
+	case Types.Backend.fsckKey backend of
+		Nothing -> return True
+		Just a -> do
+			ok <- a key file
+			unless ok $ do
+				msg <- bad key
+				warning $ "Bad file content; " ++ msg
+			return ok
 
 checkKeyNumCopies :: Key -> FilePath -> Maybe Int -> Annex Bool
 checkKeyNumCopies key file numcopies = do
@@ -299,3 +347,91 @@ badContentRemote remote key = do
 	Remote.logStatus remote key InfoMissing
 	return $ (if ok then "dropped from " else "failed to drop from ")
 		++ Remote.name remote
+
+data Incremental = StartIncremental | ContIncremental (Maybe EpochTime) | NonIncremental
+	deriving (Eq)
+
+runFsck :: Incremental -> FilePath -> Key -> Annex Bool -> CommandStart
+runFsck inc file key a = ifM (needFsck inc key)
+	( do
+		showStart "fsck" file
+		next $ do
+			ok <- a
+			when ok $
+				recordFsckTime key
+			next $ return ok
+	, stop
+	)
+
+{- Check if a key needs to be fscked, with support for incremental fscks. -}
+needFsck :: Incremental -> Key -> Annex Bool
+needFsck (ContIncremental Nothing) _ = return True
+needFsck (ContIncremental starttime) key = do
+	fscktime <- getFsckTime key
+	return $ fscktime < starttime
+needFsck _ _ = return True
+
+{- To record the time that a key was last fscked, without
+ - modifying its mtime, we set the timestamp of its parent directory.
+ - Each annexed file is the only thing in its directory, so this is fine.
+ -
+ - To record that the file was fscked, the directory's sticky bit is set.
+ - (None of the normal unix behaviors of the sticky bit should matter, so
+ - we can reuse this permission bit.)
+ -
+ - Note that this relies on the parent directory being deleted when a file
+ - is dropped. That way, if it's later added back, the fsck record
+ - won't still be present.
+ -}
+recordFsckTime :: Key -> Annex ()
+recordFsckTime key = do
+	parent <- parentDir <$> inRepo (gitAnnexLocation key)
+	liftIO $ void $ tryIO $ do
+		touchFile parent
+		setSticky parent
+
+getFsckTime :: Key -> Annex (Maybe EpochTime)
+getFsckTime key = do
+	parent <- parentDir <$> inRepo (gitAnnexLocation key)
+	liftIO $ catchDefaultIO Nothing $ do
+		s <- getFileStatus parent
+		return $ if isSticky $ fileMode s
+			then Just $ modificationTime s
+			else Nothing
+
+{- Records the start time of an interactive fsck.
+ -
+ - To guard against time stamp damange (for example, if an annex directory
+ - is copied without -a), the fsckstate file contains a time that should
+ - be identical to its modification time. -}
+recordStartTime :: Annex ()
+recordStartTime = do
+	f <- fromRepo gitAnnexFsckState
+	createAnnexDirectory $ parentDir f
+	liftIO $ do
+		nukeFile f
+		h <- openFile f WriteMode
+		t <- modificationTime <$> getFileStatus f
+		hPutStr h $ showTime $ realToFrac t
+		hClose h
+	where
+		showTime :: POSIXTime -> String
+		showTime = show
+
+resetStartTime :: Annex ()
+resetStartTime = liftIO . nukeFile =<< fromRepo gitAnnexFsckState
+
+{- Gets the incremental fsck start time. -}
+getStartTime :: Annex (Maybe EpochTime)
+getStartTime = do
+	f <- fromRepo gitAnnexFsckState
+	liftIO $ catchDefaultIO Nothing $ do
+		timestamp <- modificationTime <$> getFileStatus f
+		t <- readishTime <$> readFile f
+		return $ if Just (realToFrac timestamp) == t
+			then Just timestamp
+			else Nothing
+	where
+		readishTime :: String -> Maybe POSIXTime
+		readishTime s = utcTimeToPOSIXSeconds <$>
+			parseTime defaultTimeLocale "%s%Qs" s
