@@ -15,6 +15,7 @@ import qualified Remote
 import qualified Logs.Remote
 import qualified Types.Remote as R
 import Annex.UUID
+import Logs.UUID
 
 def :: [Command]
 def = [command "initremote"
@@ -37,18 +38,19 @@ start (name:ws) = do
 	t <- findType fullconfig
 
 	showStart "initremote" name
-	next $ perform t u $ M.union config c
+	next $ perform t u name $ M.union config c
 
 	where
 		config = Logs.Remote.keyValToConfig ws
 
-perform :: RemoteType -> UUID -> R.RemoteConfig -> CommandPerform
-perform t u c = do
+perform :: RemoteType -> UUID -> String -> R.RemoteConfig -> CommandPerform
+perform t u name c = do
 	c' <- R.setup t u c
-	next $ cleanup u c'
+	next $ cleanup u name c'
 
-cleanup :: UUID -> R.RemoteConfig -> CommandCleanup
-cleanup u c = do
+cleanup :: UUID -> String -> R.RemoteConfig -> CommandCleanup
+cleanup u name c = do
+	describeUUID u name
 	Logs.Remote.configSet u c
         return True
 
