@@ -16,6 +16,7 @@ import qualified Remote
 import Annex.UUID
 import qualified Option
 import Logs.Presence
+import Logs.Transfer
 
 def :: [Command]
 def = [withOptions options $ command "move" paramPaths seek
@@ -68,9 +69,9 @@ toStart dest move file key = do
 		then stop -- not here, so nothing to do
 		else do
 			showMoveAction move file
-			next $ toPerform dest move key
-toPerform :: Remote -> Bool -> Key -> CommandPerform
-toPerform dest move key = moveLock move key $ do
+			next $ toPerform dest move key file
+toPerform :: Remote -> Bool -> Key -> FilePath -> CommandPerform
+toPerform dest move key file = moveLock move key $ do
 	-- Checking the remote is expensive, so not done in the start step.
 	-- In fast mode, location tracking is assumed to be correct,
 	-- and an explicit check is not done, when copying. When moving,
@@ -88,17 +89,19 @@ toPerform dest move key = moveLock move key $ do
 			stop
 		Right False -> do
 			showAction $ "to " ++ Remote.name dest
-			ok <- Remote.storeKey dest key
+			ok <- upload (Remote.uuid dest) key (Just file) noRetry $
+				Remote.storeKey dest key (Just file)
 			if ok
-				then finish
+				then finish True
 				else do
 					when fastcheck $
 						warning "This could have failed because --fast is enabled."
 					stop
-		Right True -> finish
+		Right True -> finish False
 	where
-		finish = do
-			Remote.logStatus dest key InfoPresent
+		finish remotechanged = do
+			when remotechanged $
+				Remote.logStatus dest key InfoPresent
 			if move
 				then do
 					whenM (inAnnex key) $ removeAnnex key
@@ -118,7 +121,7 @@ fromStart src move file key
 	where
 		go = stopUnless (fromOk src key) $ do
 			showMoveAction move file
-			next $ fromPerform src move key
+			next $ fromPerform src move key file
 fromOk :: Remote -> Key -> Annex Bool
 fromOk src key
 	| Remote.hasKeyCheap src =
@@ -128,17 +131,17 @@ fromOk src key
 		expensive = do
 			u <- getUUID
 			remotes <- Remote.keyPossibilities key
-			return $ u /= Remote.uuid src && any (== src) remotes
-fromPerform :: Remote -> Bool -> Key -> CommandPerform
-fromPerform src move key = moveLock move key $ do
+			return $ u /= Remote.uuid src && elem src remotes
+fromPerform :: Remote -> Bool -> Key -> FilePath -> CommandPerform
+fromPerform src move key file = moveLock move key $
 	ifM (inAnnex key)
 		( handle move True
-		, do
-			showAction $ "from " ++ Remote.name src
-			ok <- getViaTmp key $ Remote.retrieveKeyFile src key
-			handle move ok
+		, handle move =<< go
 		)
 	where
+		go = download (Remote.uuid src) key (Just file) noRetry $ do
+			showAction $ "from " ++ Remote.name src
+			getViaTmp key $ Remote.retrieveKeyFile src key (Just file)
 		handle _ False = stop -- failed
 		handle False True = next $ return True -- copy complete
 		handle True True = do -- finish moving
