@@ -12,6 +12,7 @@ module Command.Status where
 import Control.Monad.State.Strict
 import qualified Data.Map as M
 import Text.JSON
+import Data.Tuple
 
 import Common.Annex
 import qualified Types.Backend as B
@@ -31,6 +32,8 @@ import Logs.Trust
 import Remote
 import Config
 import Utility.Percentage
+import Logs.Transfer
+import Types.TrustLevel
 
 -- a named computation that produces a statistic
 type Stat = StatState (Maybe (String, StatState String))
@@ -66,10 +69,11 @@ fast_stats :: [Stat]
 fast_stats = 
 	[ supported_backends
 	, supported_remote_types
-	, remote_list Trusted "trusted"
-	, remote_list SemiTrusted "semitrusted"
-	, remote_list UnTrusted "untrusted"
-	, remote_list DeadTrusted "dead"
+	, remote_list Trusted
+	, remote_list SemiTrusted
+	, remote_list UnTrusted
+	, remote_list DeadTrusted
+	, transfer_list
 	, disk_size
 	]
 slow_stats :: [Stat]
@@ -123,14 +127,14 @@ supported_remote_types :: Stat
 supported_remote_types = stat "supported remote types" $ json unwords $
 	return $ map R.typename Remote.remoteTypes
 
-remote_list :: TrustLevel -> String -> Stat
-remote_list level desc = stat n $ nojson $ lift $ do
+remote_list :: TrustLevel -> Stat
+remote_list level = stat n $ nojson $ lift $ do
 	us <- M.keys <$> (M.union <$> uuidMap <*> remoteMap Remote.name)
 	rs <- fst <$> trustPartition level us
 	s <- prettyPrintUUIDs n rs
 	return $ if null s then "0" else show (length rs) ++ "\n" ++ beginning s
 	where
-		n = desc ++ " repositories"
+		n = showTrustLevel level ++ " repositories"
 	
 local_annex_size :: Stat
 local_annex_size = stat "local annex size" $ json id $
@@ -170,6 +174,23 @@ bloom_info = stat "bloom filter size" $ json id $ do
 
 	return $ size ++ note
 
+transfer_list :: Stat
+transfer_list = stat "transfers in progress" $ nojson $ lift $ do
+	uuidmap <- Remote.remoteMap id
+	ts <- getTransfers
+	if null ts
+		then return "none"
+		else return $ multiLine $
+			map (\(t, i) -> line uuidmap t i) $ sort ts
+	where
+		line uuidmap t i = unwords
+			[ showLcDirection (transferDirection t) ++ "ing"
+			, fromMaybe (key2file $ transferKey t) (associatedFile i)
+			, if transferDirection t == Upload then "to" else "from"
+			, maybe (fromUUID $ transferUUID t) Remote.name $
+				M.lookup (transferUUID t) uuidmap
+			]
+
 disk_size :: Stat
 disk_size = stat "available local disk space" $ json id $ lift $
 	calcfree
@@ -193,10 +214,10 @@ backend_usage = stat "backend usage" $ nojson $
 		<$> (backendsKeys <$> cachedReferencedData)
 		<*> (backendsKeys <$> cachedPresentData)
 	where
-		calc a b = pp "" $ reverse . sort $ map swap $ M.toList $ M.unionWith (+) a b
-		pp c [] = c
-		pp c ((n, b):xs) = "\n\t" ++ b ++ ": " ++ show n ++ pp c xs
-		swap (a, b) = (b, a)
+		calc x y = multiLine $
+			map (\(n, b) -> b ++ ": " ++ show n) $
+			reverse $ sort $ map swap $ M.toList $
+			M.unionWith (+) x y
 
 cachedPresentData :: StatState KeyData
 cachedPresentData = do
@@ -264,3 +285,7 @@ staleSize label dirspec = go =<< lift (Command.Unused.staleKeys dirspec)
 
 aside :: String -> String
 aside s = " (" ++ s ++ ")"
+
+multiLine :: [String] -> String
+multiLine = concatMap (\l -> "\n\t" ++ l)
+
