@@ -39,12 +39,14 @@ cleanup file key = do
 	-- Commit that removal now, to avoid later confusing the
 	-- pre-commit hook if this file is later added back to
 	-- git as a normal, non-annexed file.
-	whenM (not . null <$> inRepo (LsFiles.staged [file])) $ do
+	(s, clean) <- inRepo $ LsFiles.staged [file]
+	when (not $ null s) $ do
 		showOutput
 		inRepo $ Git.Command.run "commit" [
 			Param "-q",
 			Params "-m", Param "content removed from git annex",
 			Param "--", File file]
+	void $ liftIO clean
 
 	ifM (Annex.getState Annex.fast)
 		( do
