@@ -32,7 +32,7 @@ check = do
 		error "can only run uninit from the top of the git repository"
 	where
 		current_branch = Git.Ref . Prelude.head . lines <$> revhead
-		revhead = inRepo $ Git.Command.pipeRead 
+		revhead = inRepo $ Git.Command.pipeReadStrict
 			[Params "rev-parse --abbrev-ref HEAD"]
 
 seek :: [CommandSeek]
