@@ -34,6 +34,7 @@ import qualified Remote
 import qualified Annex.Branch
 import qualified Option
 import Annex.CatFile
+import Types.Key
 
 def :: [Command]
 def = [withOptions [fromOption] $ command "unused" paramNothing seek
@@ -100,7 +101,7 @@ number n (x:xs) = (n+1, x) : number (n+1) xs
 table :: [(Int, Key)] -> [String]
 table l = "  NUMBER  KEY" : map cols l
 	where
-		cols (n,k) = "  " ++ pad 6 (show n) ++ "  " ++ show k
+		cols (n,k) = "  " ++ pad 6 (show n) ++ "  " ++ key2file k
 		pad n s = s ++ replicate (n - length s) ' '
 
 staleTmpMsg :: [(Int, Key)] -> String
@@ -227,11 +228,18 @@ withKeysReferencedM a = withKeysReferenced' () calla
 		calla k _ = a k
 
 withKeysReferenced' :: v -> (Key -> v -> Annex v) -> Annex v
-withKeysReferenced' initial a = go initial =<< files
+withKeysReferenced' initial a = do
+	(files, clean) <- getfiles
+	r <- go initial files
+	liftIO $ void clean
+	return r
 	where
-		files = do
-			top <- fromRepo Git.repoPath
-			inRepo $ LsFiles.inRepo [top]
+		getfiles = ifM isBareRepo
+			( return ([], return True)
+			, do
+				top <- fromRepo Git.repoPath
+				inRepo $ LsFiles.inRepo [top]
+			)
 		go v [] = return v
 		go v (f:fs) = do
 			x <- Backend.lookupFile f
@@ -247,14 +255,15 @@ withKeysReferencedInGit a = do
 	rs <- relevantrefs <$> showref
 	forM_ rs (withKeysReferencedInGitRef a)
 	where
-		showref = inRepo $ Git.Command.pipeRead [Param "show-ref"]
+		showref = inRepo $ Git.Command.pipeReadStrict [Param "show-ref"]
 		relevantrefs = map (Git.Ref .  snd) .
 			nubBy uniqref .
 			filter ourbranches .
 			map (separate (== ' ')) . lines
 		uniqref (x, _) (y, _) = x == y
 		ourbranchend = '/' : show Annex.Branch.name
-		ourbranches (_, b) = not $ ourbranchend `isSuffixOf` b
+		ourbranches (_, b) = not (ourbranchend `isSuffixOf` b)
+			&& not ("refs/synced/" `isPrefixOf` b)
 
 withKeysReferencedInGitRef :: (Key -> Annex ()) -> Git.Ref -> Annex ()
 withKeysReferencedInGitRef a ref = do
