@@ -56,7 +56,7 @@ remoteCost r def = do
 	cmd <- getRemoteConfig r "cost-command" ""
 	(fromMaybe def . readish) <$>
 		if not $ null cmd
-			then liftIO $ snd <$> pipeFrom "sh" ["-c", cmd]
+			then liftIO $ readProcess "sh" ["-c", cmd]
 			else getRemoteConfig r "cost" ""
 
 cheapRemoteCost :: Int
@@ -86,6 +86,11 @@ repoNotIgnored :: Git.Repo -> Annex Bool
 repoNotIgnored r = not . fromMaybe False . Git.Config.isTrue
 	<$> getRemoteConfig r "ignore" ""
 
+{- Checks if a repo should be synced. -}
+repoSyncable :: Git.Repo -> Annex Bool
+repoSyncable r = fromMaybe True . Git.Config.isTrue
+	<$> getRemoteConfig r "sync" ""
+
 {- If a value is specified, it is used; otherwise the default is looked up
  - in git config. forcenumcopies overrides everything. -}
 getNumCopies :: Maybe Int -> Annex Int
@@ -116,4 +121,4 @@ getHttpHeaders = do
 	cmd <- getConfig (annexConfig "http-headers-command") ""
 	if null cmd
 		then fromRepo $ Git.Config.getList "annex.http-headers"
-		else lines . snd <$> liftIO (pipeFrom "sh" ["-c", cmd])
+		else lines <$> liftIO (readProcess "sh" ["-c", cmd])
