@@ -9,6 +9,7 @@ module Git.Config where
 
 import qualified Data.Map as M
 import Data.Char
+import System.Process (cwd, env)
 
 import Common
 import Git
@@ -36,10 +37,13 @@ read repo@(Repo { config = c })
 
 {- Reads config even if it was read before. -}
 reRead :: Repo -> IO Repo
-reRead = read'
+reRead r = read' $ r
+	{ config = M.empty
+	, fullconfig = M.empty
+	}
 
 {- Cannot use pipeRead because it relies on the config having been already
- - read. Instead, chdir to the repo.
+ - read. Instead, chdir to the repo and run git config.
  -}
 read' :: Repo -> IO Repo
 read' repo = go repo
@@ -47,9 +51,30 @@ read' repo = go repo
 		go Repo { location = Local { gitdir = d } } = git_config d
 		go Repo { location = LocalUnknown d } = git_config d
 		go _ = assertLocal repo $ error "internal"
-		git_config d = bracketCd d $
-			pOpen ReadFromPipe "git" ["config", "--null", "--list"] $
+		git_config d = withHandle StdoutHandle createProcessSuccess p $
+			hRead repo
+			where
+				params = ["config", "--null", "--list"]
+				p = (proc "git" params)
+					{ cwd = Just d
+					, env = gitEnv repo
+					}
+
+{- Gets the global git config, returning a dummy Repo containing it. -}
+global :: IO (Maybe Repo)
+global = do
+	home <- myHomeDir
+	ifM (doesFileExist $ home </> ".gitconfig")
+		( do
+			repo <- Git.Construct.fromUnknown
+			repo' <- withHandle StdoutHandle createProcessSuccess p $
 				hRead repo
+			return $ Just repo'
+		, return Nothing
+		)
+	where
+		params = ["config", "--null", "--list", "--global"]
+		p = (proc "git" params)
 
 {- Reads git config from a handle and populates a repo with it. -}
 hRead :: Repo -> Handle -> IO Repo
@@ -68,7 +93,7 @@ hRead repo h = do
 store :: String -> Repo -> IO Repo
 store s repo = do
 	let c = parse s
-	let repo' = updateLocation $ repo
+	repo' <- updateLocation $ repo
 		{ config = (M.map Prelude.head c) `M.union` config repo
 		, fullconfig = M.unionWith (++) c (fullconfig repo)
 		}
@@ -81,16 +106,22 @@ store s repo = do
  - known. Once the config is read, this can be fixed up to a Local repo, 
  - based on the core.bare and core.worktree settings.
  -}
-updateLocation :: Repo -> Repo
+updateLocation :: Repo -> IO Repo
 updateLocation r@(Repo { location = LocalUnknown d })
-	| isBare r = newloc $ Local d Nothing
-	| otherwise = newloc $ Local (d </> ".git") (Just d)
-	where
-		newloc l = r { location = getworktree l }
-		getworktree l = case workTree r of
-			Nothing -> l
-			wt -> l { worktree = wt }
-updateLocation r = r
+	| isBare r = updateLocation' r $ Local d Nothing
+	| otherwise = updateLocation' r $ Local (d </> ".git") (Just d)
+updateLocation r@(Repo { location = l@(Local {}) }) = updateLocation' r l
+updateLocation r = return r
+
+updateLocation' :: Repo -> RepoLocation -> IO Repo
+updateLocation' r l = do
+	l' <- case getMaybe "core.worktree" r of
+		Nothing -> return l
+		Just d -> do
+			{- core.worktree is relative to the gitdir -}
+			top <- absPath $ gitdir l
+			return $ l { worktree = Just $ absPathFrom top d }
+	return $ r { location = l' }
 
 {- Parses git config --list or git config --null --list output into a
  - config map. -}
@@ -117,6 +148,3 @@ isTrue s
 
 isBare :: Repo -> Bool
 isBare r = fromMaybe False $ isTrue =<< getMaybe "core.bare" r
-
-workTree :: Repo -> Maybe FilePath
-workTree = getMaybe "core.worktree"
