@@ -5,6 +5,8 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module GitAnnex where
 
 import System.Console.GetOpt
@@ -28,6 +30,7 @@ import qualified Command.Copy
 import qualified Command.Get
 import qualified Command.FromKey
 import qualified Command.DropKey
+import qualified Command.TransferKey
 import qualified Command.ReKey
 import qualified Command.Reinject
 import qualified Command.Fix
@@ -52,13 +55,23 @@ import qualified Command.Trust
 import qualified Command.Untrust
 import qualified Command.Semitrust
 import qualified Command.Dead
+import qualified Command.Group
+import qualified Command.Ungroup
+import qualified Command.Vicfg
 import qualified Command.Sync
 import qualified Command.AddUrl
 import qualified Command.Import
 import qualified Command.Map
 import qualified Command.Upgrade
 import qualified Command.Version
+#ifdef WITH_ASSISTANT
 import qualified Command.Watch
+import qualified Command.Assistant
+#ifdef WITH_WEBAPP
+import qualified Command.WebApp
+#endif
+#endif
+import qualified Command.Help
 
 cmds :: [Command]
 cmds = concat
@@ -83,8 +96,12 @@ cmds = concat
 	, Command.Untrust.def
 	, Command.Semitrust.def
 	, Command.Dead.def
+	, Command.Group.def
+	, Command.Ungroup.def
+	, Command.Vicfg.def
 	, Command.FromKey.def
 	, Command.DropKey.def
+	, Command.TransferKey.def
 	, Command.ReKey.def
 	, Command.Fix.def
 	, Command.Fsck.def
@@ -100,7 +117,14 @@ cmds = concat
 	, Command.Map.def
 	, Command.Upgrade.def
 	, Command.Version.def
+#ifdef WITH_ASSISTANT
 	, Command.Watch.def
+	, Command.Assistant.def
+#ifdef WITH_WEBAPP
+	, Command.WebApp.def
+#endif
+#endif
+	, Command.Help.def
 	]
 
 options :: [Option]
@@ -125,9 +149,17 @@ options = Option.common ++
 		"skip files with fewer copies"
 	, Option ['B'] ["inbackend"] (ReqArg Limit.addInBackend paramName)
 		"skip files not using a key-value backend"
+	, Option [] ["inallgroup"] (ReqArg Limit.addInAllGroup paramGroup)
+		"skip files not present in all remotes in a group"
+	, Option [] ["largerthan"] (ReqArg Limit.addLargerThan paramSize)
+		"skip files larger than a size"
+	, Option [] ["smallerthan"] (ReqArg Limit.addSmallerThan paramSize)
+		"skip files smaller than a size"
+	, Option ['T'] ["time-limit"] (ReqArg Limit.addTimeLimit paramTime)
+		"stop after the specified amount of time"
 	] ++ Option.matcher
 	where
-		setnumcopies v = Annex.changeState $ \s -> s {Annex.forcenumcopies = readish v }
+		setnumcopies v = Annex.changeState $ \s -> s { Annex.forcenumcopies = readish v }
 		setgitconfig :: String -> Annex ()
 		setgitconfig v = do
 			newg <- inRepo $ Git.Config.store v
@@ -137,4 +169,4 @@ header :: String
 header = "Usage: git-annex command [option ..]"
 
 run :: [String] -> IO ()
-run args = dispatch True args cmds options header Git.CurrentRepo.get
+run args = dispatch True args cmds options [] header Git.CurrentRepo.get
