@@ -11,7 +11,6 @@ module Messages (
 	showAction,
 	showProgress,
 	metered,
-	MeterUpdate,
 	showSideAction,
 	doSideAction,
 	doQuietSideAction,
@@ -42,6 +41,7 @@ import Common
 import Types
 import Types.Messages
 import Types.Key
+import Types.Remote
 import qualified Annex
 import qualified Messages.JSON as JSON
 
@@ -63,9 +63,8 @@ showProgress = handle q $
 
 {- Shows a progress meter while performing a transfer of a key.
  - The action is passed a callback to use to update the meter. -}
-type MeterUpdate = Integer -> IO ()
-metered :: Key -> (MeterUpdate -> Annex a) -> Annex a
-metered key a = withOutputType $ go (keySize key)
+metered :: (Maybe MeterUpdate) -> Key -> (MeterUpdate -> Annex a) -> Annex a
+metered combinemeterupdate key a = withOutputType $ go (keySize key)
 	where
 		go (Just size) NormalOutput = do
 			progress <- liftIO $ newProgress "" size
@@ -75,6 +74,7 @@ metered key a = withOutputType $ go (keySize key)
 			r <- a $ \n -> liftIO $ do
 				incrP progress n
 				displayMeter stdout meter
+				maybe noop (\m -> m n) combinemeterupdate
 			liftIO $ clearMeter stdout meter
 			return r
                 go _ _ = a (const noop)
@@ -183,7 +183,7 @@ setupConsole = do
 	fileEncoding stderr
 
 handle :: IO () -> IO () -> Annex ()
-handle json normal = withOutputType $ go
+handle json normal = withOutputType go
 	where
 		go NormalOutput = liftIO normal
 		go QuietOutput = q
