@@ -20,10 +20,14 @@ module Remote (
 	remoteTypes,
 	remoteList,
 	enabledRemoteList,
+	specialRemote,
 	remoteMap,
 	uuidDescriptions,
 	byName,
+	byCost,
 	prettyPrintUUIDs,
+	prettyListUUIDs,
+	repoFromUUID,
 	remotesWithUUID,
 	remotesWithoutUUID,
 	keyLocations,
@@ -39,6 +43,7 @@ module Remote (
 import qualified Data.Map as M
 import Text.JSON
 import Text.JSON.Generic
+import Data.Tuple
 
 import Common.Annex
 import Types.Remote
@@ -48,6 +53,7 @@ import Logs.UUID
 import Logs.Trust
 import Logs.Location
 import Remote.List
+import qualified Git
 
 {- Map from UUIDs of Remotes to a calculated value. -}
 remoteMap :: (Remote -> a) -> Annex (M.Map UUID a)
@@ -75,7 +81,7 @@ byName' :: String -> Annex (Either String Remote)
 byName' "" = return $ Left "no remote specified"
 byName' n = handle . filter matching <$> remoteList
 	where
-		handle [] = Left $ "there is no git remote named \"" ++ n ++ "\""
+		handle [] = Left $ "there is no available git remote named \"" ++ n ++ "\""
 		handle match = Right $ Prelude.head match
 		matching r = n == name r || toUUID n == uuid r
 
@@ -97,7 +103,6 @@ nameToUUID n = byName' n >>= go
 				Nothing -> return $ byuuid m
 		byuuid m = M.lookup (toUUID n) $ transform double m
 		transform a = M.fromList . map a . M.toList
-		swap (a, b) = (b, a)
 		double (a, _) = (a, a)
 
 {- Pretty-prints a list of UUIDs of remotes, for human display.
@@ -128,6 +133,31 @@ prettyPrintUUIDs desc uuids = do
 			, ("here", toJSON $ hereu == u)
 			]
 
+{- List of remote names and/or descriptions, for human display.  -}
+prettyListUUIDs :: [UUID] -> Annex [String]
+prettyListUUIDs uuids = do
+	hereu <- getUUID
+	m <- uuidDescriptions
+	return $ map (\u -> prettify m hereu u) uuids
+	where
+		finddescription m u = M.findWithDefault "" u m
+		prettify m hereu u
+			| u == hereu = addName n "here"
+			| otherwise = n
+			where
+				n = finddescription m u
+
+{- Gets the git repo associated with a UUID.
+ - There's no associated remote when this is the UUID of the local repo. -}
+repoFromUUID :: UUID -> Annex (Git.Repo, Maybe Remote)
+repoFromUUID u = ifM ((==) u <$> getUUID)
+	( (,) <$> gitRepo <*> pure Nothing
+	, do
+		remote <- fromMaybe (error "Unknown UUID") . M.lookup u
+			<$> remoteMap id
+		return (repo remote, Just remote)
+	)
+
 {- Filters a list of remotes to ones that have the listed uuids. -}
 remotesWithUUID :: [Remote] -> [UUID] -> [Remote]
 remotesWithUUID rs us = filter (\r -> uuid r `elem` us) rs
@@ -208,3 +238,10 @@ forceTrust level remotename = do
  - on the remote, but this cannot always be relied on. -}
 logStatus :: Remote -> Key -> LogStatus -> Annex ()
 logStatus remote key = logChange key (uuid remote)
+
+{- Orders remotes by cost, with ones with the lowest cost grouped together. -}
+byCost :: [Remote] -> [[Remote]]
+byCost = map snd . sort . M.toList . costmap
+	where
+		costmap = M.fromListWith (++) . map costpair
+		costpair r = (cost r, [r])
