@@ -9,6 +9,7 @@ module Remote.Rsync (remote) where
 
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Map as M
+import System.Posix.Process (getProcessID)
 
 import Common.Annex
 import Types.Remote
@@ -18,7 +19,7 @@ import Annex.Content
 import Remote.Helper.Special
 import Remote.Helper.Encryptable
 import Crypto
-import Utility.RsyncFile
+import Utility.Rsync
 import Annex.Perms
 
 type RsyncUrl = String
@@ -44,21 +45,25 @@ gen r u c = do
 	return $ encryptableRemote c
 		(storeEncrypted o)
 		(retrieveEncrypted o)
-		Remote {
-			uuid = u,
-			cost = cst,
-			name = Git.repoDescribe r,
- 			storeKey = store o,
-			retrieveKeyFile = retrieve o,
-			retrieveKeyFileCheap = retrieveCheap o,
-			removeKey = remove o,
-			hasKey = checkPresent r o,
-			hasKeyCheap = False,
-			whereisKey = Nothing,
-			config = Nothing,
-			repo = r,
-			remotetype = remote
-		}
+		Remote
+			{ uuid = u
+			, cost = cst
+			, name = Git.repoDescribe r
+ 			, storeKey = store o
+			, retrieveKeyFile = retrieve o
+			, retrieveKeyFileCheap = retrieveCheap o
+			, removeKey = remove o
+			, hasKey = checkPresent r o
+			, hasKeyCheap = False
+			, whereisKey = Nothing
+			, config = Nothing
+			, repo = r
+			, localpath = if rsyncUrlIsPath $ rsyncUrl o
+				then Just $ rsyncUrl o
+				else Nothing
+			, readonly = False
+			, remotetype = remote
+			}
 
 genRsyncOpts :: Git.Repo -> Maybe RemoteConfig -> Annex RsyncOpts
 genRsyncOpts r c = do
@@ -99,17 +104,17 @@ rsyncUrls o k = map use annexHashes
 		use h = rsyncUrl o </> h k </> rsyncEscape o (f </> f)
                 f = keyFile k
 
-store :: RsyncOpts -> Key -> Annex Bool
-store o k = rsyncSend o k <=< inRepo $ gitAnnexLocation k
+store :: RsyncOpts -> Key -> AssociatedFile -> MeterUpdate -> Annex Bool
+store o k _f p = rsyncSend o p k <=< inRepo $ gitAnnexLocation k
 
-storeEncrypted :: RsyncOpts -> (Cipher, Key) -> Key -> Annex Bool
-storeEncrypted o (cipher, enck) k = withTmp enck $ \tmp -> do
+storeEncrypted :: RsyncOpts -> (Cipher, Key) -> Key -> MeterUpdate -> Annex Bool
+storeEncrypted o (cipher, enck) k p = withTmp enck $ \tmp -> do
 	src <- inRepo $ gitAnnexLocation k
 	liftIO $ withEncryptedContent cipher (L.readFile src) $ L.writeFile tmp
-	rsyncSend o enck tmp
+	rsyncSend o p enck tmp
 
-retrieve :: RsyncOpts -> Key -> FilePath -> Annex Bool
-retrieve o k f = untilTrue (rsyncUrls o k) $ \u -> rsyncRemote o
+retrieve :: RsyncOpts -> Key -> AssociatedFile -> FilePath -> Annex Bool
+retrieve o k _ f = untilTrue (rsyncUrls o k) $ \u -> rsyncRemote o Nothing
 	-- use inplace when retrieving to support resuming
 	[ Param "--inplace"
 	, Param u
@@ -117,11 +122,11 @@ retrieve o k f = untilTrue (rsyncUrls o k) $ \u -> rsyncRemote o
 	]
 
 retrieveCheap :: RsyncOpts -> Key -> FilePath -> Annex Bool
-retrieveCheap o k f = ifM (preseedTmp k f) ( retrieve o k f , return False )
+retrieveCheap o k f = ifM (preseedTmp k f) ( retrieve o k undefined f , return False )
 
 retrieveEncrypted :: RsyncOpts -> (Cipher, Key) -> Key -> FilePath -> Annex Bool
 retrieveEncrypted o (cipher, enck) _ f = withTmp enck $ \tmp -> do
-	ifM (retrieve o enck tmp)
+	ifM (retrieve o enck undefined tmp)
 		( liftIO $ catchBoolIO $ do
 			withDecryptedContent cipher (L.readFile tmp) $ L.writeFile f
 			return True
@@ -169,7 +174,7 @@ checkPresent r o k = do
  - ensure that files are only moved into place once complete
  -}
 partialParams :: CommandParam
-partialParams = Params "--no-inplace --partial --partial-dir=.rsync-partial"
+partialParams = Params "--partial --partial-dir=.rsync-partial"
 
 {- Runs an action in an empty scratch directory that can be used to build
  - up trees for rsync. -}
@@ -186,10 +191,10 @@ withRsyncScratchDir a = do
 		nuke d = liftIO $ whenM (doesDirectoryExist d) $
 			removeDirectoryRecursive d
 
-rsyncRemote :: RsyncOpts -> [CommandParam] -> Annex Bool
-rsyncRemote o params = do
+rsyncRemote :: RsyncOpts -> (Maybe MeterUpdate) -> [CommandParam] -> Annex Bool
+rsyncRemote o callback params = do
 	showOutput -- make way for progress bar
-	ifM (liftIO $ rsync $ rsyncOptions o ++ defaultParams ++ params)
+	ifM (liftIO $ (maybe rsync rsyncProgress callback) ps)
 		( return True
 		, do
 			showLongNote "rsync failed -- run git annex again to resume file transfer"
@@ -197,16 +202,17 @@ rsyncRemote o params = do
 		)
 	where
 		defaultParams = [Params "--progress"]
+		ps = rsyncOptions o ++ defaultParams ++ params
 
 {- To send a single key is slightly tricky; need to build up a temporary
    directory structure to pass to rsync so it can create the hash
    directories. -}
-rsyncSend :: RsyncOpts -> Key -> FilePath -> Annex Bool
-rsyncSend o k src = withRsyncScratchDir $ \tmp -> do
+rsyncSend :: RsyncOpts -> MeterUpdate -> Key -> FilePath -> Annex Bool
+rsyncSend o callback k src = withRsyncScratchDir $ \tmp -> do
 	let dest = tmp </> Prelude.head (keyPaths k)
 	liftIO $ createDirectoryIfMissing True $ parentDir dest
 	liftIO $ createLink src dest
-	rsyncRemote o
+	rsyncRemote o (Just callback)
 		[ Param "--recursive"
 		, partialParams
  		  -- tmp/ to send contents of tmp dir
