@@ -5,7 +5,7 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-module Remote.S3 (remote) where
+module Remote.S3 (remote, s3SetCredsEnv) where
 
 import Network.AWS.AWSConnection
 import Network.AWS.S3Object
@@ -27,6 +27,8 @@ import Remote.Helper.Encryptable
 import Crypto
 import Annex.Content
 import Utility.Base64
+import Annex.Perms
+import Utility.FileMode
 
 remote :: RemoteType
 remote = RemoteType {
@@ -60,6 +62,8 @@ gen' r u c cst =
 			whereisKey = Nothing,
 			config = c,
 			repo = r,
+			localpath = Nothing,
+			readonly = False,
 			remotetype = remote
 		}
 
@@ -83,12 +87,12 @@ s3Setup u c = handlehost $ M.lookup "host" c
 
 		use fullconfig = do
 			gitConfigSpecialRemote u fullconfig "s3" "true"
-			s3SetCreds fullconfig	
+			s3SetCreds fullconfig u
 
 		defaulthost = do
 			c' <- encryptionSetup c
 			let fullconfig = c' `M.union` defaults
-			genBucket fullconfig
+			genBucket fullconfig u
 			use fullconfig
 
 		archiveorg = do
@@ -113,14 +117,14 @@ s3Setup u c = handlehost $ M.lookup "host" c
 					-- be human-readable
 					M.delete "bucket" defaults
 
-store :: Remote -> Key -> Annex Bool
-store r k = s3Action r False $ \(conn, bucket) -> do
+store :: Remote -> Key -> AssociatedFile -> MeterUpdate -> Annex Bool
+store r k _f _p = s3Action r False $ \(conn, bucket) -> do
 	dest <- inRepo $ gitAnnexLocation k
 	res <- liftIO $ storeHelper (conn, bucket) r k dest
 	s3Bool res
 
-storeEncrypted :: Remote -> (Cipher, Key) -> Key -> Annex Bool
-storeEncrypted r (cipher, enck) k = s3Action r False $ \(conn, bucket) -> 
+storeEncrypted :: Remote -> (Cipher, Key) -> Key -> MeterUpdate -> Annex Bool
+storeEncrypted r (cipher, enck) k _p = s3Action r False $ \(conn, bucket) -> 
 	-- To get file size of the encrypted content, have to use a temp file.
 	-- (An alternative would be chunking to to a constant size.)
 	withTmp enck $ \tmp -> do
@@ -149,12 +153,12 @@ storeHelper (conn, bucket) r k file = do
 		xheaders = filter isxheader $ M.assocs $ fromJust $ config r
 		isxheader (h, _) = "x-amz-" `isPrefixOf` h
 
-retrieve :: Remote -> Key -> FilePath -> Annex Bool
-retrieve r k f = s3Action r False $ \(conn, bucket) -> do
+retrieve :: Remote -> Key -> AssociatedFile -> FilePath -> Annex Bool
+retrieve r k _f d = s3Action r False $ \(conn, bucket) -> do
 	res <- liftIO $ getObject conn $ bucketKey r bucket k
 	case res of
 		Right o -> do
-			liftIO $ L.writeFile f $ obj_data o
+			liftIO $ L.writeFile d $ obj_data o
 			return True
 		Left e -> s3Warning e
 
@@ -204,17 +208,19 @@ s3Action r noconn action = do
 	when (isNothing $ config r) $
 		error $ "Missing configuration for special remote " ++ name r
 	let bucket = M.lookup "bucket" $ fromJust $ config r
-	conn <- s3Connection $ fromJust $ config r
+	conn <- s3Connection (fromJust $ config r) (uuid r)
 	case (bucket, conn) of
 		(Just b, Just c) -> action (c, b)
 		_ -> return noconn
 
 bucketFile :: Remote -> Key -> FilePath
-bucketFile r = munge . show
+bucketFile r = munge . key2file
 	where
-		munge s = case M.lookup "mungekeys" $ fromJust $ config r of
-			Just "ia" -> iaMunge s
-			_ -> s
+		munge s = case M.lookup "mungekeys" c of
+			Just "ia" -> iaMunge $ fileprefix ++ s
+			_ -> fileprefix ++ s
+		fileprefix = M.findWithDefault "" "fileprefix" c
+		c = fromJust $ config r
 
 bucketKey :: Remote -> String -> Key -> S3Object
 bucketKey r bucket k = S3Object bucket (bucketFile r k) "" [] L.empty
@@ -231,9 +237,9 @@ iaMunge = (>>= munge)
 			| isSpace c = []
 			| otherwise = "&" ++ show (ord c) ++ ";"
 
-genBucket :: RemoteConfig -> Annex ()
-genBucket c = do
-	conn <- s3ConnectionRequired c
+genBucket :: RemoteConfig -> UUID -> Annex ()
+genBucket c u = do
+	conn <- s3ConnectionRequired c u
 	showAction "checking bucket"
 	loc <- liftIO $ getBucketLocation conn bucket 
 	case loc of
@@ -249,13 +255,13 @@ genBucket c = do
 		bucket = fromJust $ M.lookup "bucket" c
 		datacenter = fromJust $ M.lookup "datacenter" c
 
-s3ConnectionRequired :: RemoteConfig -> Annex AWSConnection
-s3ConnectionRequired c =
-	maybe (error "Cannot connect to S3") return =<< s3Connection c
+s3ConnectionRequired :: RemoteConfig -> UUID -> Annex AWSConnection
+s3ConnectionRequired c u =
+	maybe (error "Cannot connect to S3") return =<< s3Connection c u
 
-s3Connection :: RemoteConfig -> Annex (Maybe AWSConnection)
-s3Connection c = do
-	creds <- s3GetCreds c
+s3Connection :: RemoteConfig -> UUID -> Annex (Maybe AWSConnection)
+s3Connection c u = do
+	creds <- s3GetCreds c u
 	case creds of
 		Just (ak, sk) -> return $ Just $ AWSConnection host port ak sk
 		_ -> do
@@ -268,52 +274,78 @@ s3Connection c = do
 			[(p, _)] -> p
 			_ -> error $ "bad S3 port value: " ++ s
 
-{- S3 creds come from the environment if set. 
- - Otherwise, might be stored encrypted in the remote's config. -}
-s3GetCreds :: RemoteConfig -> Annex (Maybe (String, String))
-s3GetCreds c = maybe fromconfig (return . Just) =<< liftIO getenv
+{- S3 creds come from the environment if set, otherwise from the cache
+ - in gitAnnexCredsDir, or failing that, might be stored encrypted in
+ - the remote's config. -}
+s3GetCreds :: RemoteConfig -> UUID -> Annex (Maybe (String, String))
+s3GetCreds c u = maybe fromcache (return . Just) =<< liftIO getenv
 	where
 		getenv = liftM2 (,)
 			<$> get s3AccessKey
 			<*> get s3SecretKey
 			where
 				get = catchMaybeIO . getEnv
-		setenv (ak, sk) = do
-			setEnv s3AccessKey ak True
-			setEnv s3SecretKey sk True
+		fromcache = do
+			d <- fromRepo gitAnnexCredsDir
+			let f = d </> fromUUID u
+			v <- liftIO $ catchMaybeIO $ readFile f
+			case lines <$> v of
+				Just (ak:sk:[]) -> return $ Just (ak, sk)
+				_ -> fromconfig
 		fromconfig = do
 			mcipher <- remoteCipher c
 			case (M.lookup "s3creds" c, mcipher) of
-				(Just s3creds, Just cipher) ->
-					liftIO $ decrypt s3creds cipher
+				(Just s3creds, Just cipher) -> do
+					creds <- liftIO $ decrypt s3creds cipher
+					case creds of
+						[ak, sk] -> do
+							s3CacheCreds (ak, sk) u
+							return $ Just (ak, sk)
+						_ -> do error "bad s3creds"		
 				_ -> return Nothing
-		decrypt s3creds cipher = do
-			creds <- lines <$>
-				withDecryptedContent cipher
-					(return $ L.pack $ fromB64 s3creds)
-					(return . L.unpack)
-			case creds of
-				[ak, sk] -> do
-					setenv (ak, sk)
-					return $ Just (ak, sk)
-				_ -> do error "bad s3creds"
+		decrypt s3creds cipher = lines <$>
+			withDecryptedContent cipher
+				(return $ L.pack $ fromB64 s3creds)
+				(return . L.unpack)
 
-{- Stores S3 creds encrypted in the remote's config if possible. -}
-s3SetCreds :: RemoteConfig -> Annex RemoteConfig
-s3SetCreds c = do
-	creds <- s3GetCreds c
+{- Stores S3 creds encrypted in the remote's config if possible to do so
+ - securely, and otherwise locally in gitAnnexCredsDir. -}
+s3SetCreds :: RemoteConfig -> UUID -> Annex RemoteConfig
+s3SetCreds c u = do
+	creds <- s3GetCreds c u
 	case creds of
 		Just (ak, sk) -> do
 			mcipher <- remoteCipher c
 			case mcipher of
-				Just cipher -> do
+				Just cipher | isTrustedCipher c -> do
 					s <- liftIO $ withEncryptedContent cipher
 						(return $ L.pack $ unlines [ak, sk])
 						(return . L.unpack)
 					return $ M.insert "s3creds" (toB64 s) c
-				Nothing -> return c
+				_ -> do
+					s3CacheCreds (ak, sk) u
+					return c
 		_ -> return c
 
+{- The S3 creds are cached in gitAnnexCredsDir. -}
+s3CacheCreds :: (String, String) -> UUID -> Annex ()
+s3CacheCreds (ak, sk) u = do
+	d <- fromRepo gitAnnexCredsDir
+	createAnnexDirectory d
+	liftIO $ do
+		let f = d </> fromUUID u
+		h <- openFile f WriteMode
+		modifyFileMode f $ removeModes
+			[groupReadMode, otherReadMode]
+		hPutStr h $ unlines [ak, sk]
+		hClose h
+
+{- Sets the S3 creds in the environment. -}
+s3SetCredsEnv :: (String, String) -> IO ()
+s3SetCredsEnv (ak, sk) = do
+	setEnv s3AccessKey ak True
+	setEnv s3SecretKey sk True
+
 s3AccessKey :: String
 s3AccessKey = "AWS_ACCESS_KEY_ID"
 s3SecretKey :: String
