@@ -71,7 +71,7 @@ locationLogs = do
 		files <- mapM tryDirContents (concat levelb)
 		return $ mapMaybe islogfile (concat files)
 	where
-		tryDirContents d = catchDefaultIO (dirContents d) []
+		tryDirContents d = catchDefaultIO [] $ dirContents d
 		islogfile f = maybe Nothing (\k -> Just (k, f)) $
 				logFileKey $ takeFileName f
 
@@ -84,7 +84,7 @@ inject source dest = do
 
 logFiles :: FilePath -> Annex [FilePath]
 logFiles dir = return . filter (".log" `isSuffixOf`)
-		=<< liftIO (getDirectoryContents dir)
+		<=< liftIO $ getDirectoryContents dir
 
 push :: Annex ()
 push = do
