@@ -77,10 +77,16 @@ paramType :: String
 paramType = "TYPE"
 paramDate :: String
 paramDate = "DATE"
+paramTime :: String
+paramTime = "TIME"
 paramFormat :: String
 paramFormat = "FORMAT"
 paramFile :: String
 paramFile = "FILE"
+paramGroup :: String
+paramGroup = "GROUP"
+paramSize :: String
+paramSize = "SIZE"
 paramKeyValue :: String
 paramKeyValue = "K=V"
 paramNothing :: String
