@@ -15,13 +15,17 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE Rank2Types, KindSignatures #-}
+
 module Utility.Matcher (
 	Token(..),
 	Matcher,
 	token,
+	tokens,
 	generate,
 	match,
 	matchM,
+	matchMrun,
 	matchesAny
 ) where
 
@@ -48,6 +52,9 @@ token "(" = Open
 token ")" = Close
 token t = error $ "unknown token " ++ t
 
+tokens :: [String]
+tokens = words "and or not ( )"
+
 {- Converts a list of Tokens into a Matcher. -}
 generate :: [Token op] -> Matcher op
 generate = go MAny
@@ -84,13 +91,19 @@ match a m v = go m
 
 {- Runs a monadic Matcher, where Operations are actions in the monad. -}
 matchM :: Monad m => Matcher (v -> m Bool) -> v -> m Bool
-matchM m v = go m
+matchM m v = matchMrun m $ \o -> o v
+
+{- More generic running of a monadic Matcher, with full control over running
+ - of Operations. Mostly useful in order to match on more than one
+ - parameter. -}
+matchMrun :: forall o (m :: * -> *). Monad m => Matcher o -> (o -> m Bool) -> m Bool
+matchMrun m run = go m
 	where
 		go MAny = return True
 		go (MAnd m1 m2) = go m1 <&&> go m2
 		go (MOr m1 m2) =  go m1 <||> go m2
 		go (MNot m1) = liftM not (go m1)
-		go (MOp o) = o v
+		go (MOp o) = run o
 
 {- Checks is a matcher contains no limits, and so (presumably) matches
  - anything. Note that this only checks the trivial case; it is possible
