@@ -46,6 +46,9 @@ import qualified Utility.FileMode
 import qualified Utility.Gpg
 import qualified Build.SysConfig
 import qualified Utility.Format
+import qualified Utility.Verifiable
+import qualified Utility.Process
+import qualified Utility.Misc
 
 -- for quickcheck
 instance Arbitrary Types.Key.Key where
@@ -76,7 +79,7 @@ quickcheck = TestLabel "quickcheck" $ TestList
 	[ qctest "prop_idempotent_deencode_git" Git.Filename.prop_idempotent_deencode
 	, qctest "prop_idempotent_deencode" Utility.Format.prop_idempotent_deencode
 	, qctest "prop_idempotent_fileKey" Locations.prop_idempotent_fileKey
-	, qctest "prop_idempotent_key_read_show" Types.Key.prop_idempotent_key_read_show
+	, qctest "prop_idempotent_key_encode" Types.Key.prop_idempotent_key_encode
 	, qctest "prop_idempotent_shellEscape" Utility.SafeCommand.prop_idempotent_shellEscape
 	, qctest "prop_idempotent_shellEscape_multiword" Utility.SafeCommand.prop_idempotent_shellEscape_multiword
 	, qctest "prop_idempotent_configEscape" Logs.Remote.prop_idempotent_configEscape
@@ -88,6 +91,8 @@ quickcheck = TestLabel "quickcheck" $ TestList
 	, qctest "prop_hmacWithCipher_sane" Crypto.prop_hmacWithCipher_sane
 	, qctest "prop_TimeStamp_sane" Logs.UUIDBased.prop_TimeStamp_sane
 	, qctest "prop_addLog_sane" Logs.UUIDBased.prop_addLog_sane
+	, qctest "prop_verifiable_sane" Utility.Verifiable.prop_verifiable_sane
+	, qctest "prop_segment_regressionTest" Utility.Misc.prop_segment_regressionTest
 	]
 
 blackbox :: Test
@@ -174,7 +179,7 @@ test_reinject = "git-annex reinject/fromkey" ~: TestCase $ intmpclonerepo $ do
 	writeFile tmp $ content sha1annexedfile
 	r <- annexeval $ Types.Backend.getKey backendSHA1 $
 		Types.KeySource.KeySource { Types.KeySource.keyFilename = tmp, Types.KeySource.contentLocation = tmp }
-	let key = show $ fromJust r
+	let key = Types.Key.key2file $ fromJust r
 	git_annex "reinject" [tmp, sha1annexedfile] @? "reinject failed"
 	git_annex "fromkey" [key, sha1annexedfiledup] @? "fromkey failed"
 	annexed_present sha1annexedfiledup
@@ -472,33 +477,33 @@ test_unused = "git-annex unused/dropunused" ~: intmpclonerepo $ do
 	sha1annexedfilekey <- annexeval $ findkey sha1annexedfile
 	git_annex "get" [annexedfile] @? "get of file failed"
 	git_annex "get" [sha1annexedfile] @? "get of file failed"
-	checkunused []
+	checkunused [] "after get"
 	boolSystem "git" [Params "rm -q", File annexedfile] @? "git rm failed"
-	checkunused []
+	checkunused [] "after rm"
 	boolSystem "git" [Params "commit -q -m foo"] @? "git commit failed"
-	checkunused []
+	checkunused [] "after commit"
 	-- unused checks origin/master; once it's gone it is really unused
 	boolSystem "git" [Params "remote rm origin"] @? "git remote rm origin failed"
-	checkunused [annexedfilekey]
+	checkunused [annexedfilekey] "after origin branches are gone"
 	boolSystem "git" [Params "rm -q", File sha1annexedfile] @? "git rm failed"
 	boolSystem "git" [Params "commit -q -m foo"] @? "git commit failed"
-	checkunused [annexedfilekey, sha1annexedfilekey]
+	checkunused [annexedfilekey, sha1annexedfilekey] "after rm sha1annexedfile"
 
 	-- good opportunity to test dropkey also
-	git_annex "dropkey" ["--force", show annexedfilekey]
+	git_annex "dropkey" ["--force", Types.Key.key2file annexedfilekey]
 		@? "dropkey failed"
-	checkunused [sha1annexedfilekey]
+	checkunused [sha1annexedfilekey] ("after dropkey --force " ++ Types.Key.key2file annexedfilekey)
 
 	git_annex "dropunused" ["1", "2"] @? "dropunused failed"
-	checkunused []
+	checkunused [] "after dropunused"
 	git_annex "dropunused" ["10", "501"] @? "dropunused failed on bogus numbers"
 
 	where
-		checkunused expectedkeys = do
+		checkunused expectedkeys desc = do
 			git_annex "unused" [] @? "unused failed"
 			unusedmap <- annexeval $ Logs.Unused.readUnusedLog ""
 			let unusedkeys = M.elems unusedmap
-			assertEqual "unused keys differ"
+			assertEqual ("unused keys differ " ++ desc)
 				(sort expectedkeys) (sort unusedkeys)
 		findkey f = do
 			r <- Backend.lookupFile f
@@ -550,9 +555,6 @@ test_map = "git-annex map" ~: intmpclonerepo $ do
 	git_annex "describe" ["origin", "origin repo"] @? "describe 2 failed"
 	-- --fast avoids it running graphviz, not a build dependency
 	git_annex "map" ["--fast"] @? "map failed"
-	doesFileExist "map.dot" @? "map.dot not generated"
-	c <- readFile "map.dot"
-	("this repo" `isInfixOf` c && "origin repo" `isInfixOf` c) @? ("map.dot bad content: " ++ c)
 
 test_uninit :: Test
 test_uninit = "git-annex uninit" ~: intmpclonerepo $ do
@@ -696,20 +698,10 @@ git_annex command params = do
 {- Runs git-annex and returns its output. -}
 git_annex_output :: String -> [String] -> IO String
 git_annex_output command params = do
-	(frompipe, topipe) <- createPipe
-	pid <- forkProcess $ do
-		_ <- dupTo topipe stdOutput
-		closeFd frompipe
-		_ <- git_annex command params
-		exitSuccess
+	got <- Utility.Process.readProcess "git-annex" (command:params)
 	-- XXX since the above is a separate process, code coverage stats are
 	-- not gathered for things run in it.
-	closeFd topipe
-	fromh <- fdToHandle frompipe
-	got <- hGetContentsStrict fromh
-	hClose fromh
-	_ <- getProcessStatus True False pid
-	-- XXX hack Run same command again, to get code coverage.
+	-- Run same command again, to get code coverage.
 	_ <- git_annex command params
 	return got
 
@@ -842,7 +834,7 @@ checklocationlog f expected = do
 	case r of
 		Just (k, _) -> do
 			uuids <- annexeval $ Remote.keyLocations k
-			assertEqual ("bad content in location log for " ++ f ++ " key " ++ (show k) ++ " uuid " ++ show thisuuid)
+			assertEqual ("bad content in location log for " ++ f ++ " key " ++ (Types.Key.key2file k) ++ " uuid " ++ show thisuuid)
 				expected (thisuuid `elem` uuids)
 		_ -> assertFailure $ f ++ " failed to look up key"
 
@@ -877,15 +869,19 @@ unannexed = runchecks [checkregularfile, checkcontent, checkwritable]
 
 prepare :: IO ()
 prepare = do
-	-- While PATH is mostly avoided, the commit hook does run it. Make
-	-- sure that the just-built git annex is used.
+	-- While PATH is mostly avoided, the commit hook does run it,
+	-- and so does git_annex_output. Make sure that the just-built
+	-- git annex is used.
 	cwd <- getCurrentDirectory
 	p <- getEnvDefault  "PATH" ""
 	setEnv "PATH" (cwd ++ ":" ++ p) True
 	setEnv "TOPDIR" cwd True
 	-- Avoid git complaining if it cannot determine the user's email
-	-- address.
-	setEnv "EMAIL" "git-annex test <test@example.com>" True
+	-- address, or exploding if it doesn't know the user's name.
+	setEnv "GIT_AUTHOR_EMAIL" "test@example.com" True
+	setEnv "GIT_AUTHOR_NAME" "git-annex test" True
+	setEnv "GIT_COMMITTER_EMAIL" "test@example.com" True
+	setEnv "GIT_COMMITTER_NAME" "git-annex test" True
 
 changeToTmpDir :: FilePath -> IO ()
 changeToTmpDir t = do
